
# q-expansion of newform 8400.2.a.cr, downloaded from the LMFDB on 06 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 8400
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [1], [0], [1], [3], [-2], [4], [2], [-9], [-7], [-10], [-5], [-8], [-7], [-10], [-10], [6], [-8], [-13], [-1], [-4], [13], [14], [6], [2], [0], [18], [4], [-11], [13], [7], [-12], [-10], [16], [3], [-1], [4], [16], [-18], [-2], [12], [16], [8], [-19], [21], [8], [-4], [-24], [-14], [-14], [-21], [20], [-10], [-8], [-14], [17], [14], [26], [-26], [-3], [-10], [-18], [0], [8], [-14], [-7], [-11], [2], [-3], [-2], [14], [19], [4], [13], [29], [6], [17], [14], [-3], [-30], [-6], [19], [24], [-28], [26], [-16], [27], [-15], [18], [-32], [26], [-22], [29], [39], [-20], [-34], [42], [14], [-16], [-25], [35], [37], [-26], [13], [-31], [-26], [-16], [16], [-11], [0], [18], [-29], [-25], [10], [25], [33], [4], [-18], [-6], [20], [22], [-10], [10], [45], [-48], [22], [-34], [-20], [-40], [-28], [-25], [-20], [12], [47], [-32], [-36], [2], [32], [0], [23], [-42], [-30], [-29], [9], [16], [36], [40], [24], [22], [33], [-6], [-2], [-13], [-8], [52], [-3], [-51], [24], [56], [46], [-4], [-47], [24], [30], [-33], [-54], [17], [38], [-5], [-8], [5], [-38], [-15], [-26], [-32], [20], [-7], [10], [14], [-22], [-43], [-48], [39], [-54], [-9], [-30], [-51], [-38], [41], [-8], [10], [-57], [-45], [6], [33], [-8], [-19], [-37], [-22], [26], [26], [-10], [-42], [-20], [-28], [-24], [-34], [52], [-9], [52], [-69], [40], [21], [0], [64], [-48], [-23], [-70], [-33], [51], [-39], [-28], [-69], [31], [42], [-34], [28], [-71], [-32], [-45], [-61], [-46], [-4], [-61], [-46], [-22], [-64], [-3], [51], [56], [35], [10], [-66], [-21], [30], [-20], [-38], [6], [-60], [-24], [2], [16], [-48], [-56], [-42], [68], [49], [32], [-30], [62], [-40], [-15], [42], [70], [28], [32], [19], [0], [25], [11], [-52], [44], [-32], [-59], [48], [16], [-7], [34], [-29], [-24], [-56], [40], [-6], [-8], [-4], [75], [5], [57], [36], [-26], [-5], [-46], [-1], [-12], [-79], [16], [12], [74], [8], [-2], [-32], [-6], [41], [57], [17], [78], [31], [-66], [41], [38], [-36], [-54], [-39], [20], [31], [-52], [-42], [-44], [44], [-35], [44], [22], [-58], [-1], [13], [-62], [33], [46], [7], [-9], [14], [39], [62], [-84], [12], [10], [25], [78], [-26], [-92], [6], [-41], [4], [41], [-57], [-25], [-52], [-23], [-6], [-68], [-60], [-11], [42], [34], [59], [-78], [66], [52], [18], [20], [25], [-50], [32], [95], [14], [6], [30], [1], [43], [-1], [-38], [99], [50], [2], [93], [0], [71], [-40], [-34], [-80], [84], [21], [-61], [45], [-40], [20], [47], [70], [-83], [-40], [73], [46], [43], [-6], [-81], [91], [-68], [-46], [-48], [-19], [-2], [-27], [39], [-58], [78], [74], [-23], [62], [64], [18], [18], [43], [-4], [21], [91], [-74], [-24], [80], [-36], [22], [-62], [8], [-24], [80], [-86], [24], [-75], [-62], [64], [6], [-44], [3], [12], [-44], [-8], [-84], [97], [45], [24], [-16], [16], [74], [-55], [-33], [-16], [26], [106], [-24], [34], [-4], [-68], [-70], [17], [30], [-59], [15], [-15], [-90], [-74], [-43], [-45], [-43], [-113], [-94], [26], [43], [55], [-100], [2], [-46], [-23], [108], [-6], [-41], [105], [10], [-31], [-32], [52], [106], [-56], [-64], [86], [44], [-106], [55], [-3], [-78], [-30], [-48], [14], [72], [30], [-36], [72], [-42], [-64], [73], [-10], [47], [33], [-64], [-3], [0], [34], [-64], [-64], [28], [-1], [4], [-84], [-88], [-78], [-67], [0], [65], [36], [72], [45], [-12], [31], [-25], [-90], [63], [90], [-34], [116], [36], [24], [-96], [-55], [99]]
